@@ -5,5 +5,5 @@ from config import cfg
 from exceptions import NotFound, Redirect, PermanentRedirect, BasicAuth
 from routing import urls, add_apps, add_urls, url4, auto_routing
 from decorators import as_response, as_text, as_html, as_json, to_template
-from decorators import route, error, basic_auth, anticache
+from decorators import route, error, basic_auth, anticache, view
 from template import render_to_string, render_to_response
\ No newline at end of file
