@@ -7,6 +7,7 @@ import datetime
 
 from exceptions import NotFound
 from config import cfg
+import wsgi
 
 
 class UrlMap(object):
@@ -160,11 +161,14 @@ def auto_routing(rq):
     '''
     func, kwargs = urls.find(rq.path)
     if func is None:
+        if not rq.path.endswith('/'):
+            # забыли закрывающий слеш?
+            func, kwargs = urls.find(rq.path + '/')
+            if func:
+                return wsgi.redirect(rq.path + '/')        
         raise NotFound
     return func(rq, **kwargs)
     
-
-
 def add_urls(*rules):
     ''' Добавление правил списком'''
     for rule in rules:
@@ -173,7 +177,6 @@ def add_urls(*rules):
 def url4(*args, **kwargs):
     return urls.reverse(*args, **kwargs)
     
-
 def add_apps(apps):
     '''
     Добавление приложений
@@ -186,5 +189,3 @@ def add_apps(apps):
         app_name = '%s.%s' % (mod, 'views')
         __import__(app_name)
         urls.url_prefix = None
-            
-            
