@@ -17,7 +17,7 @@ def obj_from_str(str_or_obj):
     '''
     if isinstance(str_or_obj, basestring):
         if '.' not in str_or_obj:
-            str_or_obj += '.'
+            str_or_obj = str_or_obj + '.'
         mod_name, obj_name = str_or_obj.rsplit('.', 1)
         __import__(mod_name)
         mod = sys.modules[mod_name]
@@ -26,11 +26,10 @@ def obj_from_str(str_or_obj):
         return str_or_obj
 
 def list_obj_from_str(lst):
-    for i in xrange(len(lst)):
-        obj = obj_from_str(lst[i])
-        lst[i] = obj
+    for i, obj in enumerate(lst):
+        lst[i] = obj_from_str(obj)
 
-class cache_property(object):
+class cached_property(object):
     def __init__(self, f):
         self.f = f
 
